<?php
if ( wp_using_ext_object_cache() || !session_id() ) {
session_start();
if (!defined('WAFFIREWALL_SESS_STARTED')) {
        define('WAFFIREWALL_SESS_STARTED', true);
}    
}
if (!defined('ABSPATH')) exit;
/**
 * GP Firewall plugin for WordPress
 * @source      https://hub.geniusplugin.com/pub/latest/client/wordpress/gp-firewall.zip
 * @version     1.0.2
 * Version: 1.0.2
 * Author: Genius Plugin
 * Requires at least: 4.9
 * Requires PHP: 5.6
 * Plugin Name: GP Firewall
 * Description: Genius Plugin Website Firewall is the state-of-the-art WAF solution with unparalleled protection against web-based threats. Choose Genius Plugin premier WAF solution for the ultimate safeguard against web-based attacks. Experience the best-in-class protection, ensuring comprehensive shield against web-based threats and elevating your security defenses to the next level.
 * Plugin URI: https://www.geniusplugin.com
 * @license     Copyright 2024 Genius Plugin, Corp. © All Rights Reserved
 * License: Copyright 2024 Genius Plugin, Corp. © All Rights Reserved
 * License URI: https://www.geniusplugin.com
 * Text Domain: gp-firewall
 * Domain Path: /languages
 **/
if (defined('WAFFIREWALL_DISBALED')) exit;

// In a production environment, warnings should be hidden for security reasons.
// Only comment the line below if you need to.
error_reporting(E_ALL & ~E_DEPRECATED & ~E_WARNING & ~E_NOTICE & ~E_STRICT);

// Define main params
define("waffirewall_dir_url", plugin_dir_url(__FILE__));
define('WAFFIREWALL_V', '1.0.2');
define('WAFFIREWALL_ALLOW_POST', 1); // Extra data protection with an option to toggle sending POST via the plugin's firewall connector hosted code.
define('WAFFIREWALL_ALLOW_SESSION_ID', 1); // Extra data protection with an option to toggle sending SESSION ID via the plugin's firewall connector hosted code.
define('WAFFIREWALL_ALLOW_HEADERS', 1); // Extra data protection with an option to toggle sending HEADERS via the plugin's firewall connector hosted code.
define('WAF_update_url', 'https://hub.geniusplugin.com/pub/wp_version.php');
define('WAF_APP_NAME', 'w');
// Load plugin text domain for translations
load_plugin_textdomain('gp-firewall', false, dirname(plugin_basename(__FILE__)) . '/languages/');

function waffirewall_namespace_costum_assets()
{
    wp_enqueue_style('waffirewall_costum_assets_css', plugins_url('assets/css/style.css', __FILE__), array(),  WAFFIREWALL_V);
    wp_enqueue_script('waffirewall_costum_assets_js', plugins_url('assets/js/script.js', __FILE__), array(),  WAFFIREWALL_V, true);
}

/**
 * Include pages and create object for each class
 */
function waffirewall_admin_pages()
{
    $settings = get_option('_waffirewall');
    if (isset($settings['waffirewall_TOKEN']) && !empty($settings['waffirewall_TOKEN']) && (!isset($settings['waffirewall_HUB']) || empty($settings['waffirewall_HUB'])))
    waffirewall_getTower();
    require_once('dashboard.php');
    require_once('settings.php');
    new WAFFIREWALL_SettingsPage();
}


/**
 * Get the active screen/page
 * @return string
 */
function waffirewall_toggle_menu_display()
{
    if (is_admin()) {
        $my_current_screen = get_current_screen();
        if ($my_current_screen->base == 'settings_page_waffirewall-setting') {
            add_filter('submenu_file', 'waffirewall_highlight_menu', 10, 0);
        }
    }
    return $my_current_screen;
}

/** 
 * Remove all actions hooked to the 'admin_notices' hook.
 */
function waffirewall_hide_other_notices()
{
    global $wp_filter;

    // Check if the WP_Admin_Bar exists, as it's not available on all admin pages.
    if (isset($wp_filter['admin_notices'])) {
        unset($wp_filter['admin_notices']);
    }
}

/**
 * Hides the left side menu on ip tables display so we have the largest view to table
 */
function waffirewall_hide_menu($classes)
{
    return $classes . " folded";
}

/**
 * Highlights the setting page left-menu when setting page is active
 * @return string
 */
function waffirewall_highlight_menu()
{
    return 'waffirewall-setting'; // set it active
}

/**
 * Adds a help tab on screen top, when page loads
 * @return void
 */
function waffirewall_add_help_tab()
{
    $settings = get_option('_waffirewall');
    $screen = get_current_screen();

    // Add help tab if current screen is WAF
    $screen->add_help_tab(array(
        "id"    => "waffirewall_help_tab",
        "title" => __("Help", 'gp-firewall'), 
        "content"   => "<p>" . __("
        <h3>About</h3>
        <div class='justify'>To manage your firewall, please log in to your <a href='" . $settings['waffirewall_HUB'] . "' target=_blank>dashboard</a>.
        <p>GP Firewall is the First WAF that won't take down your website if it goes offline, as it's neither DNS/CDN-based nor hosted on the same server. It operates independently, ensuring continuous protection without impacting site availability.</p>
        </div>
            ",
            'gp-firewall' 
        ) . "</p>",
        'callback' => false,
        'priority' => 10
    ));
    
}

/**
 * Create needed table at plugin load
 * @return void
 */
function waffirewall_load_plugin()
{

}

/**
 * Define a constant if ever the plugin is desactivated. Will be used to clinch plugin status in other pages.
 * @return void
 */
function waffirewall_unload_plugin()
{
    define('WAFFIREWALL_DISBALED', 'WAFFIREWALL_DISBALED');
}

/**
 * Checks that website uses php v5.3.6 or later or exit
 * @return void
 */
function waffirewall_install_notify()
{
    // Check php version
    if (version_compare(esc_html(PHP_VERSION), '5.3.6', '<')) {
        printf(
            /* translators: %s: PHP version */
            esc_html__('This plugin cannot run on PHP version: %s. Please upgrade to 5.3.6 or later', 'gp-firewall'),
            esc_html(PHP_VERSION)
        );
        return;
    }
}



add_action('admin_menu', 'waffirewall_add_admin_menu');

function waffirewall_add_admin_menu() {
    // Create a top-level menu
    add_menu_page(
        'GP Firewall', 
        'GP Firewall', 
        'manage_options', 
        'waffirewall-setting', 
        'waffirewall_display_settings', 
        'dashicons-shield-alt', 
        21 
    );
}

function waffirewall_display_settings() {   
    require_once plugin_dir_path(__FILE__) . 'settings.php'; 
    
}

/**
 * Function to Crypt data
 *
 * @param [void] $data
 * @param [string] $key
 * @return string
 */
function waffirewall_encryptClientData($data, $key)
{
    $iv = openssl_random_pseudo_bytes(openssl_cipher_iv_length('AES-256-CBC'));
    $encrypted = openssl_encrypt($data, 'AES-256-CBC', $key, 0, $iv);
    $encoded = base64_encode($encrypted . '::' . $iv);
    return $encoded;
}

/**
 * Get application and plugins name, version, status and slug
 * @return string
 */
function waffirewall_app_version()
{

    // Get WordPress Core Version
    $waffirewall_APPV['Core'] = get_bloginfo('version');

    if (!function_exists('get_plugins')) {
        include_once ABSPATH . 'wp-admin/includes/plugin.php';
    }

    
    $all_plugins = get_plugins();
    foreach ($all_plugins as $plugin_file => $plugin_info) {
        $plugin_slug = basename(dirname($plugin_file));
        // We recommend checking all plugins, but clients can opt to check only active ones.
        // Uncomment the following lines to include only active plugins.

        // Check if the current plugin is active // uncomment this to check only active plugins
        // $active_plugins = get_option('active_plugins', array());
        // if (in_array($plugin_file, $active_plugins)) { // uncomment this to check only active plugins
            // Construct the plugin version information string
            $waffirewall_APPV[$plugin_info['Name']] = $plugin_info['Version'] . ';' . $plugin_slug . ';' . 'null';
        // } // uncomment this to check only active plugins
    }

    return wp_json_encode($waffirewall_APPV);
}


/**
 * Main Firewall header modify and implementation. Changes the header of any page, sends request to tower then allow/deny access to user.
 * @param int $side
 * @return void
 */

function waffirewall_header_modify()
{
   
    if (!is_admin()) {
        $side = 1;
    } else {
        $side = -1;
    }
    // Get the value of the setting we've registered with register_setting()
    $settings = get_option('_waffirewall');
    $dismissedKeys = array();
    $dismissedHeaders = array();
    // Exit if no token, no tower or both website sides disabled
    if (!isset($settings['waffirewall_TOKEN']) || empty($settings['waffirewall_TOKEN'])) return;
    if (!isset($settings['waffirewall_TOWER']) || empty($settings['waffirewall_TOWER'])) return;
    if ($side == 1 && $settings['waffirewall_FRONT'] != 'on') return;
    if ($side == -1 && $settings['waffirewall_BACK'] != 'on') return;
    $settings['waffirewall_SIDE'] = $side;
    $_SESSION['WAF_TOKEN'] = $settings['waffirewall_TOKEN'];
    // refresh servers
    if ($settings['waffirewall_CHECKPOINT'] != gmdate('YmdH') . substr(gmdate('i'), 0, 1)) {
        if ((gmdate('YmdH') . substr(gmdate('i'), 0, 1)) % 2 == 0)
        waffirewall_getTower();
        $settings = get_option('_waffirewall');
    }


    if (isset($settings['waffirewall_XFRAME']) && !empty($settings['waffirewall_XFRAME']) && $settings['waffirewall_XFRAME'] != -1) {
        $XFRAME = json_decode(base64_decode($settings['waffirewall_XFRAME']), true);
        if (is_array($XFRAME)) {
            foreach ($XFRAME as $key => $value) {
                if ($key === 'X-Powered-By') {
                    header_remove('X-Powered-By');
                } elseif ($key === 'Server') {
                    header_remove('Server');
                } elseif ($key === 'enforceSessionTimeout') {
                    enforceSessionTimeout($value);
                } elseif ($key === 'enforceAbsoluteTimeout') {
                    enforceAbsoluteTimeout($value);
                } elseif ($key === 'bindSessionToIP') {
                    bindSessionToIP();
                } elseif ($key === 'bindSessionToUserAgent') {
                    bindSessionToUserAgent();
                } elseif ($key === 'privatepost') {
                    $dismissedKeys = explode(',',base64_decode($value));
                } elseif ($key === 'privateheader') {
                    $dismissedHeaders = explode(',',base64_decode($value));
                } else {
                    header($key . ": " . $value);
                }
            }
        }
    }



    if (isset($_POST['uniquehash']) && !empty($_POST['uniquehash'])) {
        waffirewall_checkHash(wp_json_encode($_POST), $settings['waffirewall_TOWER']);
        exit;
    }


    $test_ip = '';
    $agent = '';

    // Catch true ip
    $user_ip = isset($_SERVER['REMOTE_ADDR']) ? trim($_SERVER['REMOTE_ADDR']) : '';
    $server_ip = isset($_SERVER['SERVER_ADDR']) ? $_SERVER['SERVER_ADDR'] : '';

    if (isset($_SERVER['HTTP_Forwarded-For']) && !empty($_SERVER['HTTP_Forwarded-For'])) {
        $_SERVER['HTTP_Forwarded-For'] = isset($_SERVER['HTTP_Forwarded-For']) ? trim(current(explode(',', $_SERVER['HTTP_Forwarded-For']))) : '';
        if (filter_var($_SERVER['HTTP_Forwarded-For'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_FORWARDED_FOR']) ? trim($_SERVER['HTTP_FORWARDED_FOR']) : '';
        }
    }

    if (isset($_SERVER['HTTP_X-Client-IP']) && !empty($_SERVER['HTTP_X-Client-IP'])) {
        $_SERVER['HTTP_X-Client-IP'] = isset($_SERVER['HTTP_X-Client-IP']) ? trim(current(explode(',', $_SERVER['HTTP_X-Client-IP']))) : '';
        if (filter_var($_SERVER['HTTP_X-Client-IP'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_X-Client-IP']) ? trim($_SERVER['HTTP_X-Client-IP']) : '';
        }
    }

    if (isset($_SERVER['HTTP_X-Cluster-Client-Ip']) && !empty($_SERVER['HTTP_X-Cluster-Client-Ip'])) {
        $_SERVER['HTTP_X-Cluster-Client-Ip'] = isset($_SERVER['HTTP_X-Cluster-Client-Ip']) ? trim(current(explode(',', $_SERVER['HTTP_X-Cluster-Client-Ip']))) : '';
        if (filter_var(
            $_SERVER['HTTP_X-Cluster-Client-Ip'],
            FILTER_VALIDATE_IP,
            FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE
        )) {
            $user_ip = isset($_SERVER['HTTP_X-Cluster-Client-Ip']) ? trim($_SERVER['HTTP_X-Cluster-Client-Ip']) : '';
        }
    }

    if (isset($_SERVER['HTTP_Client-IP']) && !empty($_SERVER['HTTP_Client-IP'])) {
        $_SERVER['HTTP_Client-IP'] = isset($_SERVER['HTTP_Client-IP']) ? trim(current(explode(',', $_SERVER['HTTP_Client-IP']))) : '';
        if (filter_var($_SERVER['HTTP_Client-IP'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_Client-IP']) ? trim($_SERVER['HTTP_Client-IP']) : '';
        }
    }

    if (isset($_SERVER['HTTP_CF_CONNECTING_IP']) && !empty($_SERVER['HTTP_CF_CONNECTING_IP'])) {
        $_SERVER['HTTP_CF_CONNECTING_IP'] = isset($_SERVER['HTTP_CF_CONNECTING_IP']) ? trim(current(explode(',', $_SERVER['HTTP_CF_CONNECTING_IP']))) : '';
        if (filter_var($_SERVER['HTTP_CF_CONNECTING_IP'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_CF_CONNECTING_IP']) ? trim($_SERVER['HTTP_CF_CONNECTING_IP']) : '';
        }
    }

    if (isset($_SERVER['HTTP_X_REAL_IP']) && !empty($_SERVER['HTTP_X_REAL_IP'])) {
        $_SERVER['HTTP_X_REAL_IP'] = isset($_SERVER['HTTP_X_REAL_IP']) ? trim(current(explode(',', $_SERVER['HTTP_X_REAL_IP']))) : '';
        if (filter_var($_SERVER['HTTP_X_REAL_IP'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_X_REAL_IP']) ? trim($_SERVER['HTTP_X_REAL_IP']) : '';
        }
    }

    if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && !empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
        $_SERVER['HTTP_X_FORWARDED_FOR'] = isset($_SERVER['HTTP_X_FORWARDED_FOR']) ? trim(current(explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']))) : '';
        if (filter_var($_SERVER['HTTP_X_FORWARDED_FOR'], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
            $user_ip = isset($_SERVER['HTTP_X_FORWARDED_FOR']) ? trim($_SERVER['HTTP_X_FORWARDED_FOR']) : '';
        }
    }
    if ($user_ip == $server_ip || $user_ip == '::1' || $user_ip == '127.0.0.1' || $user_ip == $_SERVER['SERVER_ADDR']  || empty($user_ip)) {
        return;
    }


    // catch test ip
    if (isset($_GET['ip']) && !empty($_GET['ip'])) {
        $test_ip = isset($_GET['ip']) ? $_GET['ip'] : '';
    }
    // catch true url
    $protocol = 'http';
    if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 443) $protocol = 'https';
    if (isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] == 443) $protocol = 'https';
    if (isset($_SERVER['HTTP_X_FORWARDED_PORT']) && $_SERVER['HTTP_X_FORWARDED_PORT'] == 443) $protocol = 'https';
    if (isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') $protocol = 'https';
    $page = isset($_SERVER['HTTP_HOST']) && isset($_SERVER['REQUEST_URI']) ? esc_url($protocol . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']) : '';
    if (isset($_SERVER['HTTP_USER_AGENT'])) {
        $agent = isset($_SERVER['HTTP_USER_AGENT']) ? base64_encode($_SERVER['HTTP_USER_AGENT']) : '';
    }
    if ($settings['waffirewall_POST'] == 1 && WAFFIREWALL_ALLOW_POST != 0) {
        $filteredPost = waffirewall_FilterPostData();


        /*
        User can specify which posts are inspected for malicious activity. 
        It is recommended to check all POST requests for comprehensive protection. However, the user has the option to specify which POST requests are excluded based on their specific security needs.

        $dismissedKeys[] = 'key1';
        */
        foreach ($dismissedKeys as $dismissedKey) {
            unset($filteredPost[$dismissedKey]);
        }

        $sendrequests =  waffirewall_encryptClientData(base64_encode(wp_json_encode($filteredPost)), $settings['waffirewall_TOKEN']);
    } else {
        $sendrequests = '';
    }
    if ($settings['waffirewall_SESSION'] == 1 && WAFFIREWALL_ALLOW_SESSION_ID != 0) {
        if (version_compare(PHP_VERSION, '7.0.0', '>=')) {
            setcookie('session_cookie', 'value', ['expires' => time() + 3600, 'path' => '/', 'domain' => '', 'secure' => true, 'httponly' => true, 'samesite' => 'Lax']);
        } else {
            setcookie('session_cookie', 'value', time() + 3600, '/', '', true, true);
        }

        $sessionid =  base64_encode(session_id());
    } else {
        $sessionid = '';
    }
    if ($settings['waffirewall_HEADERS'] == 1 && WAFFIREWALL_ALLOW_HEADERS != 0) {
        $filteredServer = getallheaders();
        if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
            $languages = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);
            foreach ($languages as $lang) {
                $lang = explode(';', $lang)[0];
                if (preg_match('/^([a-zA-Z\-]+)/', $lang, $matches)) {
                    $preferred_language = $matches[1];
                    break;
                }
            }
        }
        if (isset($preferred_language)) {
            $filteredServer['Detected-Language'] = $preferred_language;
        }else{
            $filteredServer['Detected-Language'] = 'ND';
        }
        // Calculate the size of the POST data string
        $filteredServer['Post-Size'] = waffirewall_GetPostSize();

        /* 
        User can specify which headers are inspected for malicious activity.
        It is recommended to check all headers for comprehensive protection. However, the user has the option to specify which headers are excluded based on their specific security needs.
        $dismissedHeaders[] = 'HTTP_USER_AGENT';
        */
        foreach ($dismissedHeaders as $dismissedHeader) {
            unset($filteredServer[$dismissedHeader]);
        }
        $waffirewall_headers = waffirewall_encryptClientData(base64_encode(wp_json_encode($filteredServer)), $settings['waffirewall_TOKEN']);
    } else {
        $waffirewall_headers = '';
    }

    $data = array(
        'token' => $settings['waffirewall_TOKEN'],
        'ip' => $user_ip,
        'side' => $side,
        'tip' => $test_ip,
        'ag' => $agent,
        'page' => base64_encode($page),
        'requests' => $sendrequests,
        'session' => $sessionid,
        'headers' => $waffirewall_headers,
        'wafapp' => waffirewall_app_version()
    );
    $url =  $settings['waffirewall_TOWER'] . '?v=' . WAFFIREWALL_V . '&app=w&token=' . $settings['waffirewall_TOKEN'];
    $responseText = waffirewall_postData($url, $data);

    if (isset($_SESSION['WAF_update_run']) && $_SESSION['WAF_update_run'] === true) {
        try {
            $_SESSION['WAF_AUTOUPDATE'] = true;
            waffirewall_CheckForUpdates(); // Trigger the update check
            // global $WAF_update_log; print_r($WAF_update_log); // Error Logs for support
        } catch (Exception $e) {
            // No error message displayed for security reasons.
        }
    }

    if (!empty($responseText)) {


        $response = $responseText;
        // Send email if user asked for alert on denied access
        if (strpos($responseText, '-WAFMAIL-') !== false) {
            $response = explode('-WAFMAIL-', $responseText)[0];
            $mailer = explode('-WAFMAIL-', $responseText)[1];
            $mailer = json_decode(base64_decode($mailer));
            mail($mailer[0], $mailer[1], $mailer[2]);
        }
        // Refresh servers on demand
        if (strpos($responseText, '-WAFREDIRECT-') !== false) {
            $response = explode('-WAFREDIRECT-', $responseText)[0];
            waffirewall_getTower();
        }
        // Deny access to the page
        if ($response == 'deny') {
            exit;
        } elseif ($response == '501') {
            // Disable the firewall
            $settings['waffirewall_FRONT'] = null;
            $settings['waffirewall_BACK'] = null;
            update_option('_waffirewall', $settings);
            return;
        } else {
            // Process firewall options
            echo $response;
            exit;
        }
    }
}

/**
 * Filter POST data based on size and content.
 * @return array Returns an array of limited POST data size
 */
function waffirewall_FilterPostData() {
    $size_mega = 1; // manually change it to decimal up to 8
    $max_total_size = $size_mega * 1024 * 1024; 
    $filtered_post = [];
    $total_size = 0;

    foreach ($_POST as $key => $value) {
         if (!is_scalar($value)) {
            continue;
        }
        $keySize = strlen($key);
        $value = strval($value);
        $valueSize = strlen($value);
        $pairSize = $keySize + $valueSize;

        // Check if adding the new key-value pair would exceed the maximum total size
        if (($total_size + $pairSize) > $max_total_size) {
            break;
        }
        $filtered_post[$key] = $value;
        $total_size += $pairSize; // Update total size with the size of the key-value pair
    }

    return $filtered_post;
}

/*
  Return the total size of all key-value pairs
*/
function waffirewall_GetPostSize() {
    $total_size = 0;

    foreach ($_POST as $key => $value) {
        if (is_scalar($value)) {
            $keySize = strlen($key);
            $value = strval($value);
            $valueSize = strlen($value);
            $pairSize = $keySize + $valueSize; 
            $total_size += $pairSize; 
        }
    }

    return $total_size; 
}



function waffirewall_checkHash($hashes, $tower)
{
    $url = $tower . '?app=w&v=' . WAFFIREWALL_V;
    $data['hashes'] = $hashes;
    $responseText = waffirewall_postData($url, $data);
    if (!empty($responseText)) {
        echo $responseText;
        exit;
    }
    return;
}

register_activation_hook(__FILE__, 'waffirewall_load_plugin');
register_deactivation_hook(__FILE__, 'waffirewall_unload_plugin');
add_action('admin_init', 'waffirewall_namespace_costum_assets');
add_action('admin_notices', 'waffirewall_install_notify');
add_action('current_screen', 'waffirewall_toggle_menu_display');
if (is_admin()) {
    add_action('init', 'waffirewall_admin_pages');
} 

// Protect pages, provide side
add_action('wp-head', 'waffirewall_header_modify', 1, 1);
if (!is_admin()) {
    do_action('wp-head', 1);
} else {
    do_action('wp-head', -1);
}



/*
    * Get (new) tower and/or (new) dashboard set for the license. 
    * @return void
    * 
    */
function waffirewall_getTower($auto = 'refresh')
{
    // Get the value of the setting we've registered with register_setting()
    $settings = get_option('_waffirewall');
    if (!isset($settings['waffirewall_TOKEN']) || empty($settings['waffirewall_TOKEN'])) return;
    $_SESSION['WAF_TOKEN'] = $settings['waffirewall_TOKEN'];
    $url =  'https://hub.geniusplugin.com/license.php?v=' . WAFFIREWALL_V . '&app=w&action=gettower&token=' . $settings['waffirewall_TOKEN'];
    $result = waffirewall_fetchData($url);

    if (!empty($result)) {
        $server = json_decode($result);
        if (is_array($server)) {
            $settings['waffirewall_TOWER'] = $server[0];
            $settings['waffirewall_HUB'] = $server[1];
            $settings['waffirewall_CHECKPOINT'] = gmdate('YmdH') . substr(gmdate('i'), 0, 1);
            $settings['waffirewall_CSRF'] = $server[3];
            $settings['waffirewall_XFRAME'] = $server[4];
            $settings['waffirewall_POST'] = $server[5];
            $settings['waffirewall_SESSION'] = $server[6];
            $settings['waffirewall_HEADERS'] = $server[7];
            if (isset($server[8]) && strpos($server[8], '-WAFMAIL-') === 0) {
                $mailer = explode('-WAFMAIL-', $server[8])[1];
                $mailer = json_decode(base64_decode($mailer));
                mail($mailer[0], $mailer[1], $mailer[2]);
            }
            if (isset($server[9]) && strpos($server[9], '-WAFUPDATE-') === 0) {
                $_SESSION['WAFUPDATE'] = true; // transient
                $_SESSION['WAF_AUTOUPDATE'] = true;
                // Trigger the update check waffirewall_CheckForUpdates(); 
                $_SESSION['WAF_update_run'] = true;
            } else {
                $_SESSION['WAF_AUTOUPDATE'] = false;
                $_SESSION['WAFUPDATE'] = false; // transient
            }
            if ($auto == 'refresh') update_option('_waffirewall', $settings);
            return $server;
        }
    }
    $settings['waffirewall_CHECKPOINT'] = gmdate('YmdH') . substr(gmdate('i'), 0, 1);
    if ($auto === 'refresh') update_option('_waffirewall', $settings);
    return null;
}


/* Enforces session timeout by checking last activity and destroying session if expired
 * @param void
 * @return void
 */
function enforceSessionTimeout($timeoutDuration = 1800) {
    if (isSessionStarted()) {

        if (!isset($_SESSION['LAST_ACTIVITY'])) {
            $_SESSION['LAST_ACTIVITY'] = time();
        }

        if (time() - $_SESSION['LAST_ACTIVITY'] > $timeoutDuration) {
            session_unset();
            session_destroy();

            if (isset($_COOKIE[session_name()])) {
                setcookie(session_name(), '', time() - 3600, '/');
            }

            $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
            $mainDomain = "$protocol://" . $_SERVER['HTTP_HOST'];
            $currentUrl = "$protocol://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
            
            if ($currentUrl !== $mainDomain . '/' && $currentUrl !== $mainDomain) {
                header("Location: $mainDomain");
                exit();
            }
        }

        $_SESSION['LAST_ACTIVITY'] = time();
    }
}


/* Enforces an absolute timeout by destroying session after a set duration, regardless of activity
 * @param void
 * @return void
 */
function enforceAbsoluteTimeout($absoluteDuration = 7200) {
    if (isSessionStarted()) {
        if (!isset($_SESSION['CREATED'])) {
            $_SESSION['CREATED'] = time();
        } elseif (time() - $_SESSION['CREATED'] > $absoluteDuration) {
            session_unset();
            session_destroy();

            if (isset($_COOKIE[session_name()])) {
                setcookie(session_name(), '', time() - 3600, '/');
            }

            $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
            $mainDomain = "$protocol://" . $_SERVER['HTTP_HOST'];
            $currentUrl = "$protocol://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
            
            if ($currentUrl !== $mainDomain . '/' && $currentUrl !== $mainDomain) {
                header("Location: $mainDomain");
                exit();
            }
        }
    }
}


/* Binds session to the user's IP address to prevent hijacking from different IPs
 * @param void
 * @return void
 */
function bindSessionToIP() {
    if (isSessionStarted()) {
        if (!isset($_SESSION['IP_ADDRESS'])) {
            $_SESSION['IP_ADDRESS'] = $_SERVER['REMOTE_ADDR'];
        } elseif ($_SESSION['IP_ADDRESS'] !== $_SERVER['REMOTE_ADDR']) {
            session_unset();
            session_destroy();

            if (isset($_COOKIE[session_name()])) {
                setcookie(session_name(), '', time() - 3600, '/');
            }

            $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
            $mainDomain = "$protocol://" . $_SERVER['HTTP_HOST'];
            $currentUrl = "$protocol://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
            
            if ($currentUrl !== $mainDomain . '/' && $currentUrl !== $mainDomain) {
                header("Location: $mainDomain");
                exit();
            }
        }
    }
}


/* Binds session to the user's User-Agent to prevent hijacking from different devices or browsers
 * @param void
 * @return void
 */
function bindSessionToUserAgent() {
    if (isSessionStarted()) {
        if (!isset($_SESSION['USER_AGENT'])) {
            $_SESSION['USER_AGENT'] = $_SERVER['HTTP_USER_AGENT'];
        } elseif ($_SESSION['USER_AGENT'] !== $_SERVER['HTTP_USER_AGENT']) {
            session_unset();
            session_destroy();

            if (isset($_COOKIE[session_name()])) {
                setcookie(session_name(), '', time() - 3600, '/');
            }

            $protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http';
            $mainDomain = "$protocol://" . $_SERVER['HTTP_HOST'];
            $currentUrl = "$protocol://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
            
            if ($currentUrl !== $mainDomain . '/' && $currentUrl !== $mainDomain) {
                header("Location: $mainDomain");
                exit();
            }
        }
    }
}


/* Checks if a session has already started to avoid multiple starts
 * @param void
 * @return bool
 */
function isSessionStarted() {
    if (version_compare(phpversion(), '5.4.0', '>=')) {
        return session_status() === PHP_SESSION_ACTIVE;
    } else {
        return isset($_SESSION) && count($_SESSION) > 0;
    }
}


/* outputs notification message
 * @param mixed $data
 * @return void
 */

function waffirewall_output_notify($data)
{
    if (!empty($data)) {
        echo base64_decode($data);
    }
}


function waffirewall_gprequest()
{
    $cache_allowed = false;
    $remote = get_transient('waffirewall_ckey');

    if (false === $remote || !$cache_allowed) {
        $appVersion = get_bloginfo('version');
        $remote = wp_remote_get(
            'https://hub.geniusplugin.com/pub/wp_version.json?appversion='.$appVersion,
            array(
                'timeout' => 3,
                'headers' => array(
                    'Accept' => 'application/json'
                )
            )
        );

        if (is_wp_error($remote) || 200 !== wp_remote_retrieve_response_code($remote) || empty(wp_remote_retrieve_body($remote))) {
            return false;
        }

        set_transient('waffirewall_ckey', $remote, DAY_IN_SECONDS);
    }

    $remote = json_decode(wp_remote_retrieve_body($remote));

    return $remote;
}


function waffirewall_gpinfo($res, $action, $args)
{
    if ('plugin_information' !== $action) {
        return $res;
    }

    if (plugin_basename(__DIR__) !== $args->slug) {
        return $res;
    }

    $remote = waffirewall_gprequest();

    if (!$remote) {
        return $res;
    }

    $res = new stdClass();

    $res->name = $remote->name;
    $res->slug = $remote->slug;
    $res->version = $remote->version;
    $res->tested = $remote->tested;
    $res->requires = $remote->requires;
    $res->author = $remote->author;
    $res->author_profile = $remote->author_profile;
    $res->download_link = $remote->download_url;
    $res->trunk = $remote->download_url;
    $res->requires_php = $remote->requires_php;
    $res->last_updated = $remote->last_updated;

    $res->sections = array(
        'description' => $remote->sections->description,
        'installation' => $remote->sections->installation,
        'changelog' => $remote->sections->changelog
    );

    if (!empty($remote->banners)) {
        $res->banners = array(
            'low' => $remote->banners->low,
            'high' => $remote->banners->high
        );
    }

    return $res;
}

function waffirewall_gpupdate($transient)
{
    waffirewall_getTower();
    if (!isset($_SESSION['WAFUPDATE']) || $_SESSION['WAFUPDATE'] !== true) {
        return null;
    }

    if (empty($transient->checked)) {
        return $transient;
    }

    $remote = waffirewall_gprequest();

    $res = new stdClass();
    if ($remote && version_compare(WAFFIREWALL_V, $remote->version, '<') && version_compare($remote->requires, get_bloginfo('version'), '<=') && version_compare($remote->requires_php, PHP_VERSION, '<')) {

        $res->slug = plugin_basename(__DIR__);
        $res->plugin = plugin_basename(__FILE__);
        $res->new_version = $remote->version;
        $res->tested = $remote->tested;
        $res->package = $remote->download_url;

        $transient->response[$res->plugin] = $res;
    }

    return $transient;
}

function waffirewall_gppurge($upgrader, $options)
{
    $cache_allowed = false;

    if ($cache_allowed && 'update' === $options['action'] && 'plugin' === $options['type']) {
        delete_transient('waffirewall_ckey');
    }
}

function waffirewall_postData($url, $data) {

    $args = array(
        'timeout' => 3,
        'sslverify' => false,
        'headers' => array(
            'Content-Type' => 'application/x-www-form-urlencoded',
            'Accept-Encoding' => 'gzip'
        ),
        'follow_location' => true,
        'redirection' => 3,
        'body' => $data,
        'method' => 'POST'
    );
    
    $response = wp_remote_post($url, $args);

    if (is_wp_error($response)) {
        return false;
    }

    return wp_remote_retrieve_body($response);
}

function waffirewall_fetchData($url)
{
    global $WAF_update_log;
     // Use cURL or file_get_contents to fetch data from a URL
     if (function_exists('curl_init')) {
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1.5);
        curl_setopt($ch, CURLOPT_TIMEOUT, 3);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
        $data = curl_exec($ch);

        if (curl_errno($ch)) {
            if (function_exists('file_get_contents')) {
                return @file_get_contents($url);
            } else {
                $WAF_update_log .= ' Failed to fetch data from a URL 1';
                return false;
            }
        }
        curl_close($ch);
        return $data;
    } elseif (function_exists('file_get_contents')) {
        return @file_get_contents($url);
    } else {
        $WAF_update_log .= ' Failed to fetch data from a URL 2';
        return false;
    }
}


add_filter(
    'plugin_row_meta',
    function ($links_array, $plugin_file_name, $plugin_data, $status) {

        if (strpos($plugin_file_name, basename(__FILE__))) {

            $links_array[] = sprintf(
                '<a href="%s" class="thickbox open-plugin-details-modal">%s</a>',
                add_query_arg(
                    array(
                        'tab' => 'plugin-information',
                        'plugin' => plugin_basename(__DIR__),
                        'TB_iframe' => true,
                        'width' => 772,
                        'height' => 788
                    ),
                    admin_url('plugin-install.php')
                ),
                __('View details')
            );
        }

        return $links_array;
    },
    25,
    4
);

function waffirewall_CheckForUpdates()
{
    global $WAF_update_log;
    $check_update_core_functions = waffirewall_checkRequiredCoreFunctions();
    if ($check_update_core_functions === null) {
        return;
    }

    if ($_SESSION['WAF_update_performed']) {
        $WAF_update_log .= ' Update was already performed';
        return; // Skip update check if an update was already performed
    }

    // Check if an update has been performed in this session
    if (isset($_SESSION['update_performed']) && $_SESSION['update_performed'] === true) {
        $WAF_update_log .= ' Update was already performed in this session';
        return; // Skip update check if an update was already performed
    }

    // Fetch the version information from the server
    $versionInfo = waffirewall_fetchVersionInfo();

    // Check if version information is available
    if (!$versionInfo) {
        $WAF_update_log .= ' Version information is not available';
        return;
    }

    // Check if an update is required
    if (waffirewall_isUpdateRequired($versionInfo)) {
        // Perform the update process
        waffirewall_performUpdate($versionInfo);
    }
}

function waffirewall_fetchVersionInfo()
{
    global $WAF_update_log;
    $appVersion = get_bloginfo('version');
    // Attempt to fetch version information using file_get_contents
    $versionInfoJson = waffirewall_fetchData(WAF_update_url . '?v='.WAFFIREWALL_V . '&app=' . WAF_APP_NAME.'&token='.$_SESSION['WAF_TOKEN'].'&appversion='.$appVersion);
    if ($versionInfoJson === false) {
        $WAF_update_log .= ' Fetch version information using file_get_contents failed';
        return false;
    }


    // Decode the fetched JSON version information
    $versionInfo = json_decode($versionInfoJson, true);

    // Check if the version information is valid
    if (!is_array($versionInfo) || !isset($versionInfo['new_version'], $versionInfo['download_uri'], $versionInfo['sha256'], $versionInfo['auto_update'], $versionInfo['min_php_version'], $versionInfo['min_script_version'])) {
        $WAF_update_log .= ' Version information invalid';
        return false;
    }
    if (empty($versionInfo['new_version']) || empty($versionInfo['download_uri']) || empty($versionInfo['sha256']) || empty($versionInfo['auto_update']) || empty($versionInfo['min_php_version']) || empty($versionInfo['min_script_version'])) {
        $WAF_update_log .= ' Version information invalid';
        return false;
    }
    return $versionInfo;
}

function waffirewall_isUpdateRequired($versionInfo)
{
    global $WAF_update_log;
    // Check if auto-update is enabled
    if ($versionInfo['auto_update'] !== 1) {
        $WAF_update_log .= ' Auto-update is disabled';
        return false;
    }

    // Check if current PHP version meets the minimum required version
    if (version_compare(PHP_VERSION, $versionInfo['min_php_version'], '<')) {
        $WAF_update_log .= ' PHP version does not meet the minimum required version';
        return false;
    }

    // Check if current script meets the minimum required version for upgrade
    if (version_compare(WAFFIREWALL_V, $versionInfo['min_script_version'], '<')) {
        $WAF_update_log .= ' Script version does not meet the minimum required version';
        return false;
    }

    // Check if current script version is older than the new version
    if (version_compare($versionInfo['new_version'], WAFFIREWALL_V, '<')) {
        $WAF_update_log .= ' New version is not greater than current version';
        return false;
    }

    // Check if an autoupdate is set by user
    if ($_SESSION['WAF_AUTOUPDATE'] !== true) {
        $WAF_update_log .= ' Autoupdate is disabled';
        return false;
    }

    // Verify SHA256
    return  waffirewall_verifySha256($versionInfo['download_uri'], $versionInfo['sha256']);
}

function waffirewall_performUpdate($versionInfo)
{
    global $WAF_update_log;
    // Download the updated zip file
    $updatedZip = @file_get_contents($versionInfo['download_uri']);

    if ($updatedZip === false) {
        // file_get_contents function alternative
        // Download the updated zip file using cURL
        $updatedZip = waffirewall_downloadWithCurl($versionInfo['download_uri']);
        if (!$updatedZip) {
            $WAF_update_log .= ' Download the updated zip file failed';
            return;
        }
    }


    // Create a backup directory and copy all files and folders for non-wordpress plugin
    $backupDir = __DIR__ . '/backup_';
    waffirewall_removeDirectory($backupDir);
    if (WAF_APP_NAME != 'w') {
        if ($_SESSION['WAF_update_backup_created'] !== true) {
            if (@mkdir($backupDir)) {
                // Backup directory created successfully
                $_SESSION['WAF_update_backup_created'] = true;

                // Copy all files and folders
                if (waffirewall_copyDirectoryContents(__DIR__, $backupDir) !== true) {
                    return;
                }
            } elseif (@is_dir($backupDir)) {
                // Backup directory created successfully
                $_SESSION['WAF_update_backup_created'] = true;

                // Copy all files and folders
                if (waffirewall_copyDirectoryContents(__DIR__, $backupDir) !== true) {
                    return;
                }
            } else {
                // Failed to create backup directory
                $WAF_update_log .= ' Failed to create backup directory';
                return;
            }
        }
    }


    $tempDir = __DIR__ . '/temp_';
    waffirewall_removeDirectory($tempDir);
    // Create a temporary directory for extraction
    if ($_SESSION['WAF_update_tmp_created'] !== true) {
        if (@mkdir($tempDir)) {
            // Temporary directory created successfully
            $_SESSION['WAF_update_tmp_created'] = true;

            // Attempt to extract the updated files using PHP's ZipArchive
            $extracted = waffirewall_extractZipWithPHP($updatedZip, $tempDir, $versionInfo);

            // If extraction using PHP fails, try using PclZip
            if (!$extracted) {
                $extracted = waffirewall_extractZipWithPclZip($updatedZip, $tempDir, $versionInfo);
                if (!$extracted) {
                    $WAF_update_log .= ' Failed to extract the zip file';
                    return;
                }
            }
        } elseif (@is_dir($backupDir)) {
            // Temporary directory created successfully
            $_SESSION['WAF_update_tmp_created'] = true;

            // Attempt to extract the updated files using PHP's ZipArchive
            $extracted = waffirewall_extractZipWithPHP($updatedZip, $tempDir, $versionInfo);

            // If extraction using PHP fails, try using PclZip
            if (!$extracted) {
                $extracted = waffirewall_extractZipWithPclZip($updatedZip, $tempDir, $versionInfo);
                if (!$extracted) {
                    $WAF_update_log .= ' Failed to extract the zip file';
                    return;
                }
            }
        } else {
            // Failed to create temporary directory
            $WAF_update_log .= ' Failed to create temporary directory';
            return;
        }
    }

    // Copy the updated files to the main directory
    if (waffirewall_copyDirectoryContents($tempDir, dirname(__DIR__)) !== true) {
        return;
    }
    waffirewall_removeDirectory($tempDir); // Remove the temporary directory

    // If a backup directory exists, restore files from backup for non-wordpress plugin
    if (WAF_APP_NAME != 'w') {
        if (file_exists($backupDir)) {
            if (!isset($_SESSION['update_performed'])  || $_SESSION['update_performed'] !== true) {
                waffirewall_copyDirectoryContents($backupDir, __DIR__); // Restore from backup
            }
            waffirewall_removeDirectory($backupDir); // Remove the backup directory
        }
    }
    @unlink(__DIR__ . '/update.' . $versionInfo['new_version'] . '.zip'); // remove the zip file
    $_SESSION['WAF_update_performed'] = false;
    unset($_SESSION['update_performed']);
    unset($_SESSION['update_performed']);
    unset($_SESSION['WAF_update_tmp_created']);
    unset($_SESSION['WAF_update_backup_created']);
    unset($_SESSION['WAF_update_run']);

    // Require the updated script file & Refresh the class definition
    // require_once __FILE__;
    // new gpirewall($this->WAF_SIDE, $this->WAF_TOKEN);
}

function waffirewall_copyDirectoryContents($source, $destination)
{
    global $WAF_update_log;
    // Copy all files and folders from source to destination
    $dir = opendir($source);
    // Check if the destination directory is writable
    if (!is_writable($destination)) {
        $WAF_update_log .=  " Destination directory is not writable: $destination";
        return false;
    }

    $counter = 30; // Set initial counter value
    while (($file = readdir($dir)) !== false && $counter > 0) {
        if ($file != '.' && $file != '..') {
            $src = $source . '/' . $file;
            $dst = $destination . '/' . $file;

            // Skip copying the backup directory
            if (@is_dir($src) && strpos($file, 'backup_') === 0) {
                continue; // Skip copying backup_ folders
            }
            // Skip copying hidden directories
            if (@is_dir($src) && substr($file, 0, 1) === '.') {
                continue; // Skip copying hidden directories
            }

            if (@is_dir($src)) {
                if (!@mkdir($dst) && !@is_dir($dst)) {
                    $WAF_update_log .=  " Failed to create directory: $dst";
                    return false;
                }
                waffirewall_copyDirectoryContents($src, $dst);
            } else {
                if (!@copy($src, $dst)) {
                    $WAF_update_log .=  " Failed to copy file: $src to $dst";
                    return false;
                }
            }
        }
        $counter--; // Decrement the counter
    }

    closedir($dir);
    if (strpos($source, 'temp_') !== false) {
        //After performing the update set the flag to true
        $_SESSION['WAF_update_performed'] = true;
        $_SESSION['update_performed'] = true;
    }
    return true;
}

function waffirewall_extractZipWithPHP($zipData, $destination, $versionInfo)
{
    global $WAF_update_log;
    // Create a unique zip file name based on the version
    $zipFileName = $destination . '/update.' . $versionInfo['new_version'] . '.zip';

    // Attempt to use file_put_contents to save the zip file
    if (!file_put_contents($zipFileName, $zipData)) {
        // If file_put_contents fails, use an alternative method (downloadWithCurl)
        $downloadedZip = waffirewall_downloadWithCurl($versionInfo['download_uri']);
        if ($downloadedZip !== false) {
            // Attempt to save the downloaded zip data using fopen, fwrite, and fclose
            $zipFileHandle = fopen($zipFileName, 'w');
            if ($zipFileHandle !== false) {
                fwrite($zipFileHandle, $downloadedZip);
                fclose($zipFileHandle);
            }
        }
    }

    // Use PHP's ZipArchive to extract the files
    $zip = new ZipArchive();

    if ($zip->open($zipFileName) === true) {
        $zip->extractTo($destination);
        $zip->close();
        return true;
    } else {
        $WAF_update_log .= ' Failed to extract the zip file';
        return false;
    }
}


function waffirewall_downloadWithCurl($url)
{
    global $WAF_update_log;
    // Download data using cURL
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 1.5);
    curl_setopt($ch, CURLOPT_TIMEOUT, 3);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_MAXREDIRS, 3);
    $data = curl_exec($ch);

    if (curl_errno($ch)) {
        $WAF_update_log .= ' Failed to download data with curl';
        return false;
    }

    curl_close($ch);
    return $data;
}

function waffirewall_extractZipWithPclZip($zipData, $destination, $versionInfo)
{
    global $WAF_update_log;
    require_once 'pclzip.php';  // Include the PclZip library
    // Create a unique zip file name based on the version
    $zipFileName = $destination . '/update.' . $versionInfo['new_version'] . '.zip';
    // Need file_put_contents function alternative
    file_put_contents($zipFileName, $zipData);

    // Use PclZip library to extract the files
    $zip = new PclZip($zipFileName);
    $extractionResult = $zip->extract(PCLZIP_OPT_PATH, $destination);

    if ($extractionResult === 0) {
        $WAF_update_log .= ' Failed to Use PclZip library to extract the files';
        return false;
    }

    return true;
}

function waffirewall_removeDirectory($path)
{
    // Remove a directory and its contents
    if (@is_dir($path)) {
        $objects = @scandir($path);

        foreach ($objects as $object) {
            if ($object != '.' && $object != '..') {
                if (@is_dir($path . '/' . $object)) {
                    waffirewall_removeDirectory($path . '/' . $object);
                } else {
                    @unlink($path . '/' . $object);
                }
            }
        }

        @rmdir($path);
    }
}


function waffirewall_verifySha256($fileUri, $expectedSha256)
{
    global $WAF_update_log;
    $downloadedFile = @file_get_contents($fileUri);

    if ($downloadedFile === false) {
        // file_get_contents function alternative
        $downloadedFile = waffirewall_downloadWithCurl($fileUri);
        if ($downloadedFile === false) {
            $WAF_update_log .= ' Failed to download with curl the zip file';
            return false;
        }
    }

    // Calculate SHA256 hash and compare
    $calculatedSha256 = hash('sha256', $downloadedFile);

    if ($calculatedSha256 !== $expectedSha256) {
        $WAF_update_log .= ' Failed zip file checksum verification';
    }
    return $calculatedSha256 === $expectedSha256;
}

function waffirewall_checkRequiredCoreFunctions()
{
    $requiredFunctions = array('rmdir', 'scandir', 'file_put_contents', 'mkdir', 'unlink', 'copy', 'is_dir');

    foreach ($requiredFunctions as $function) {
        if (!function_exists($function)) {
            return null;
        }
    }

    return true; // All required functions are available
}



add_filter('plugins_api', 'waffirewall_gpinfo', 20, 3);
add_filter('site_transient_update_plugins', 'waffirewall_gpupdate');
add_action('upgrader_process_complete', 'waffirewall_gppurge', 10, 2);
session_write_close();