<?php
if ( wp_using_ext_object_cache() || !session_id() ) {
session_start();
if (!defined('WAFFIREWALL_SESS_STARTED')) {
    define('WAFFIREWALL_SESS_STARTED', true);
}    
}
if (!defined('ABSPATH')) exit;
/**
 * GP Firewall plugin for WordPress
 * source:      https://hub.geniusplugin.com/pub/latest/client/wordpress/gp-firewall.zip
 * Version: 1.0.2
 * License: Copyright 2024 Genius Plugin, Corp. © All Rights Reserved
 **/
if (defined('WAFFIREWALL_DISBALED') || !defined('WAFFIREWALL_V')) exit;

class WAFFIREWALL_SettingsPage
{
    /**
     * Holds the values to be used in the fields callbacks 
     */
    private $options;

    /**
     * Start up
     */
    public function __construct()
    {
        add_action('admin_menu', array($this, 'waffirewall_add_plugin_page'));
        add_action('admin_init', array($this, 'waffirewall_check_option'));
        add_action('admin_init', array($this, 'waffirewall_page_init'));
    }

    /**
     * Add options page
     */
    public function waffirewall_add_plugin_page()
    {
        // This page will be under "Settings"
        $waffirewall_setting_page = add_options_page(
            'Settings Admin',
            'GP Firewall',
            'manage_options',
            'waffirewall-setting',
            array($this, 'waffirewall_create_admin_page'),
            1
        );

        // Adds help tab when this page loads
        add_action('load-' . $waffirewall_setting_page,  'waffirewall_hide_other_notices');
        add_action('load-' . $waffirewall_setting_page, 'waffirewall_add_help_tab');
    }

    /**
     * Automatic license check
     * @return string
     */
    public function waffirewall_check_license($token)
    {
        $settings = get_option('_waffirewall');
        $url =  $settings['waffirewall_HUB'] . '/license.php?v=' . WAFFIREWALL_V . '&app=w&action=licensecheck&token=' . $token;
        $check_result = $this->waffirewall_fetchData($url);

        return $check_result;
    }


    public function waffirewall_fetchData($url) {
        // Use wp http api or file_get_contents to fetch data from a URL
        $args = array(
            'timeout' => 3,
            'sslverify' => false,
            'headers' => array(
                'Accept-Encoding' => 'gzip'
            ),
            'follow_location' => true,
            'redirection' => 3,
            'connect_timeout' => 1.5
        );
        
        $response = wp_remote_get($url, $args);

        return wp_remote_retrieve_body($response);
    }
    /**
     * Create the admin page
     */
    public function waffirewall_create_admin_page()
    {
        // check user capabilities
        if (!current_user_can('manage_options')) {
            return;
        }

        // Get options 
        $this->options = get_option('_waffirewall');

        // Set active tab 
        if (!isset($_GET['waffirewall_nonce_get_page']) || !wp_verify_nonce($_GET['waffirewall_nonce_get_page'], 'waffirewall_nonce_action_get_page')) {
            $active_tab = 'waffirewall-setting'; 
        }else{
            if (isset($_GET['page'])) {
                $active_tab = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
            } else {
                $active_tab = 'waffirewall-setting'; 
            }
        }

        global $waffirewallCheck;
        if (isset($this->options['waffirewall_TOKEN']) && !empty($this->options['waffirewall_TOKEN'])) {
            // Get result of license request
            $waffirewallCheck = json_decode($this->waffirewall_check_license($this->options['waffirewall_TOKEN']), true);
        } else {
            $waffirewallCheck = -1;
        }

        // Set the Waf status
        if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200 && isset($waffirewallCheck[1]) && !empty($waffirewallCheck[1])) {
            $firewall_Status = $waffirewallCheck[1];
        } else {
            $firewall_Status = 'inactive';
        }

        // Notify user if asked for by the plugin.
        if (isset($waffirewallCheck[4]) && $waffirewallCheck[4] == 1) {

            // Notify once per session, n times remotely
            if (!isset($_SESSION['waffirewall_NOTIFIER']) || $_SESSION['waffirewall_NOTIFIER'] != gmdate('Ymd')) {
                $_SESSION['waffirewall_NOTIFIER'] = gmdate('Ymd');
                add_action('admin_notices', $this->waffirewall_notify_user());
            }
        }

        ?>
        <script>jQuery(function($) {$('li a[href="options-general.php?page=waffirewall-setting"]').parent().remove();});</script>
        <table style="width:100%">
            <td>
                <h2><img src="<?php echo esc_url(waffirewall_dir_url); ?>/assets/img/waffirewall-black.png" class="waffirewall_title"> Genius Plugin Firewall</h2>
            </td>
            </tr>
            <tr>
                <td colspan="2">
                    <?php
                    // Display current version or request upgrade
                    if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200 && isset($waffirewallCheck[3]) && !empty($waffirewallCheck[3])) {
                        echo esc_html($waffirewallCheck[3]) . '<hr>';
                    }
                    // Display a map of firewall hits only to limited IPs specified in the admin's allowed ruleset short list (20 allowed ips), upon validation of a valid token
                    if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200 && (!empty($this->options['waffirewall_FRONT']) || !empty($this->options['waffirewall_BACK']))) {
                    ?>
                        <div style="height:50vh;overflow:hidden !important"><iframe style="width:100%;min-height:80vh;" src="<?php echo esc_url($this->options['waffirewall_HUB'] . '/noc-r.php?app=w&token=' . $this->options['waffirewall_TOKEN']); ?>" sandbox="allow-same-origin allow-scripts"></iframe></div>
                    <?php
                    }
                    ?>
                </td>
            </tr>
            <tr>
                <td colspan="2">
                    <?php
                    // Display first activation welcome message
                    if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200 && isset($waffirewallCheck[2]) && !empty($waffirewallCheck[2])) {
                        echo '<div id="setting-error-settings_updated" class="notice notice-info settings-info is-dismissible"><p><strong> ' . esc_html($waffirewallCheck[2]) . '</strong></p></div>';
                    }

                    // Display warning message on empty token or invalid token
                    if ($waffirewallCheck == -1 || $waffirewallCheck[0] != 200) {
                        $this->options['waffirewall_FRONT'] = null;
                        $this->options['waffirewall_BACK'] = null;
                        update_option('_waffirewall', $this->options);
                        echo '<hr><div id="setting-error-settings_updated" class="notice notice-warning settings-error is-dismissible">
                              <p>License token is invalid. Please fill the below field with your GP Firewall token, then clic on the submit button. <a href="https://cloud.geniusplugin.com/order.php?step=1&productGroup=1" target="_blank">Need a free license token?</a><br>
                              </strong></p></div>';
                    }
                    // Display warning message if silence is the only thing we get from remote server
                    if (!is_array($waffirewallCheck) && $waffirewallCheck != -1) {
                        $this->options['waffirewall_FRONT'] = null;
                        $this->options['waffirewall_BACK'] = null;
                        update_option('_waffirewall', $this->options);
                        echo '<div id="setting-error-settings_updated" class="notice notice-warning settings-error is-dismissible"><p><strong>There was a temporary problem. Please check your token, then try again or contact support.</strong></p></div>';
                    }

                    ?>
                </td>
            </tr>
        </table>
        <?php // Display tabs
        if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200) { $get_page_nonce = wp_create_nonce('waffirewall_nonce_action_get_page'); ?>
            <div class="wrap">
                <hr>
                <h2 class="nav-tab-wrapper">
                    <a href="options-general.php?page=waffirewall-setting&waffirewall_nonce_get_page=<?php echo esc_attr($get_page_nonce); ?>" class="nav-tab <?php echo $active_tab == 'waffirewall-setting' ? 'nav-tab-active' : ''; ?>">License</a>
                    <a href="<?php echo esc_url($this->options['waffirewall_HUB']) ?>" target="_blank" class="nav-tab <?php echo $active_tab == 'waffirewall-iptables' ? 'nav-tab-active' : ''; ?>">Dashboard</a>
                </h2>
            <?php } ?>
            <form method="post" id="tokenupdate" action="options.php">
                <?php
                // Print out all hidden setting fields
                settings_fields('waffirewall_option_group');
                do_settings_sections('waffirewall-setting');
                submit_button();
                ?>
            </form>
            



    <?php
    }

    /*
    * Output informations on dashboard top; hits limit notification here
    * @return void
    * 
    */
    function waffirewall_notify_user($dashboard = 0)
    {
        $settings = get_option('_waffirewall');
        $token = apply_filters('waffirewall_TOKEN', $settings['waffirewall_TOKEN']);
        $response = wp_remote_get($settings['waffirewall_HUB'] . '/license.php?v=' . WAFFIREWALL_V . '&app=w&section=' . $dashboard . '&action=pluginboard&token=' . $token, array('timeout' => 3));

        $http_code = wp_remote_retrieve_response_code($response);
        if ($http_code == 200) {
            $body = wp_remote_retrieve_body($response);

            waffirewall_output_notify($body);
        } else {
            waffirewall_output_notify($http_code);
            return;
        }
    }


    function waffirewall_check_option() {
        // Retrieve the _waffirewall option
        $waffirewall_options = get_option('_waffirewall', array());
    
        // Define the default array
        $default_values = array(
            'waffirewall_TOKEN' => '',
            'waffirewall_FRONT' => '',
            'waffirewall_BACK' => '',
        );
    
        // Check if _waffirewall option exists and its values
        if (empty($waffirewall_options) || !is_array($waffirewall_options)) {
            // If _waffirewall option does not exist or is not an array, set it to default values
            update_option('_waffirewall', $default_values);
        } 
    }


    /**
     * Register and add settings
     */
    public function waffirewall_page_init()
    {
        register_setting(
            'waffirewall_option_group',
            '_waffirewall',
            array($this, 'waffirewall_sanitize')
        );
        add_settings_section(
            'setting_section_id',
            '',
            array($this, 'waffirewall_add_setting_section_callback'),
            'waffirewall-setting'
        );
        add_settings_field(
            'waffirewall_TOKEN',
            'License Token',
            array($this, 'waffirewall_TOKEN_callback'),
            'waffirewall-setting',
            'setting_section_id'
        );
        add_settings_field(
            'waffirewall_FRONT',
            'Front Side',
            array($this, 'waffirewall_FRONT_callback'),
            'waffirewall-setting',
            'setting_section_id'
        );
        add_settings_field(
            'waffirewall_BACK',
            'Back side',
            array($this, 'waffirewall_BACK_callback'),
            'waffirewall-setting',
            'setting_section_id'
        );
       
    }

    /**
     * sanitize the token and remove any potentially harmful content
     *
     * @param string $input
     * @return boolean
     */
    function waffirewall_isValidToken($input)
    {
        if (strlen($input) > 42) {
            return false;
        }
        $sanitizedInput = htmlspecialchars($input, ENT_QUOTES, 'UTF-8');
        if ($sanitizedInput !== $input) {
            return false;
        }
        if (preg_match('/<[^>]*>/', $input)) {
            return false;
        }
        $pattern = '/^[a-zA-Z0-9]+(?:-[a-zA-Z0-9]+){0,8}$/';
        return preg_match($pattern, $input) === 1;
    }

    /**
     * Sanitize each setting field as needed. 
     *
     * @param array $input Contains all settings fields as array keys
     */
    public function waffirewall_sanitize($input)
    {
        if (!isset($input['waffirewall_TOKEN']) || !$this->waffirewall_isValidToken($input['waffirewall_TOKEN'])) return null;

        $new_input = array();
        $new_input['waffirewall_TOKEN'] = sanitize_text_field($input['waffirewall_TOKEN']);
        if (isset($input['waffirewall_FRONT'])) $new_input['waffirewall_FRONT'] =  'on';
        if (isset($input['waffirewall_BACK'])) $new_input['waffirewall_BACK'] =  'on';

        $servers = waffirewall_getTower('norefresh');
        if ($servers != null && is_array($servers)) {
            $new_input['waffirewall_TOWER'] = esc_url($servers[0]);
            $new_input['waffirewall_HUB'] = esc_url($servers[1]);
            $new_input['waffirewall_CHECKPOINT'] = gmdate('YmdH') . substr(gmdate('i'), 0, 1);
            $new_input['waffirewall_CSRF'] = $servers[3];
            $new_input['waffirewall_XFRAME'] = $servers[4];
            $new_input['waffirewall_POST'] = $servers[5];
            $new_input['waffirewall_SESSION'] = $servers[6];
            $new_input['waffirewall_HEADERS'] = $servers[7];
        } 

        if (!isset($input['waffirewall_CSRF']))
            $new_input['waffirewall_CSRF'] =  -1;
        if (!isset($input['waffirewall_XFRAME']))
            $new_input['waffirewall_XFRAME'] =  -1;
        if (!isset($input['waffirewall_POST']))
            $new_input['waffirewall_POST'] =  -1;
        if (!isset($input['waffirewall_SESSION']))
            $new_input['waffirewall_SESSION'] =  -1;
        if (!isset($input['waffirewall_HEADERS']))
            $new_input['waffirewall_HEADERS'] =  -1;
        

        return $new_input;
    }


    public function waffirewall_add_setting_section_callback()
    {
        //
    }

    public function waffirewall_TOKEN_callback()
    {
        global $waffirewallCheck;
        $readonly = "";
        if (is_array($waffirewallCheck) && $waffirewallCheck[0] == 200) {
            $readonly = " readonly ";
        }
        
        printf(
            '<input type="text" id="waffirewall_TOKEN" %s class="w60vw" name="_waffirewall[waffirewall_TOKEN]" value="%s" /><br><b>On license change, both Front and Back Side must be checked.</b><hr>',
            esc_attr($readonly),
            esc_attr(isset($this->options['waffirewall_TOKEN']) ? $this->options['waffirewall_TOKEN'] : '')
        );
    }

    function waffirewall_FRONT_callback()
    {
        global $waffirewallCheck;
        if (isset($this->options['waffirewall_FRONT']) && is_array($waffirewallCheck) && $waffirewallCheck[0] == 200) {
            $frontchecked = 'checked';
        }
        printf(
            '<input type="checkbox" id="waffirewall_FRONT" class="w60vw" name="_waffirewall[waffirewall_FRONT]" value="%s" %s /> Protect the front side  (Firewall must be active from the dashboard)<hr>',
            esc_attr(isset($this->options['waffirewall_FRONT']) ? 'on' : ''),
            esc_attr(isset($frontchecked) ? $frontchecked : '')
        );
        
    }

    function waffirewall_BACK_callback()
    {

        if (isset($this->options['waffirewall_BACK'])) {
            $backchecked = 'checked';
        }
        
        printf(
            '<input type="checkbox" id="waffirewall_BACK" class="w60vw" name="_waffirewall[waffirewall_BACK]" value="%s" %s /> Protect the back side  (Firewall must be active from the dashboard)<hr>',
            esc_attr(isset($this->options['waffirewall_BACK']) ? 'on' : ''),
            esc_attr(isset($backchecked) ? $backchecked : '')
        );
    }

}
?>